"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperationExecutionError = exports.PublicError = void 0;
class PublicError extends Error {
    constructor(message, cause) {
        if (cause instanceof Error && !message) {
            super(cause.message);
            this.stack = this.stack + '\nCaused By: ' + cause.stack;
        }
        else if (arguments.length <= 1) {
            super(message);
        }
        else {
            super(message);
            if (cause instanceof Error) {
                this.stack = this.stack + '\nCaused By: ' + cause.stack;
            }
            else {
                this.stack = this.stack + '\nCaused By: ' + cause;
            }
        }
        Object.defineProperty(this, 'cause', {
            value: cause,
            writable: true,
            enumerable: false,
            configurable: true
        });
    }
}
exports.PublicError = PublicError;
PublicError.isPublicError = (error) => {
    return error instanceof PublicError;
};
class OperationExecutionError extends Error {
    constructor(operation, context, operationType, message, cause) {
        super(OperationExecutionError.createErrorMessage(operation, context, operationType, message, cause));
        let simpleMessage;
        if (cause instanceof Error && !message) {
            this.stack = this.stack + '\nCaused By: ' + cause.stack;
            if (cause instanceof OperationExecutionError) {
                simpleMessage = cause.simpleMessage;
            }
            else {
                simpleMessage = cause.message;
            }
        }
        else if (arguments.length <= 4) {
            if (message) {
                simpleMessage = message + '';
            }
            else {
                simpleMessage = '';
            }
        }
        else {
            if (cause instanceof Error) {
                this.stack = this.stack + '\nCaused By: ' + cause.stack;
            }
            else {
                this.stack = this.stack + '\nCaused By: ' + cause;
            }
            if (message) {
                simpleMessage = message + '';
            }
            else if (cause) {
                simpleMessage = cause + '';
            }
            else {
                simpleMessage = '';
            }
        }
        Object.defineProperty(this, 'cause', {
            value: cause,
            writable: true,
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(this, 'simpleMessage', {
            value: simpleMessage,
            writable: true,
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(this, 'operationType', {
            value: operationType,
            writable: true,
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(this, 'operation', {
            value: operation,
            writable: true,
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(this, 'context', {
            value: context,
            writable: true,
            enumerable: false,
            configurable: true
        });
    }
    sameContent(operation, context, operationType) {
        return this.operationType === operationType && this.operation === operation && this.context === context;
    }
}
exports.OperationExecutionError = OperationExecutionError;
OperationExecutionError.stringifyContext = (operation, _operationType) => {
    return JSON.stringify(operation);
};
OperationExecutionError.stringifyOperation = (operation, _operationType) => {
    return JSON.stringify(operation);
};
OperationExecutionError.createErrorMessage = _createOperationExecutionErrorMessage;
function _createOperationExecutionErrorMessage(operation, context, operationType, message, cause) {
    let result = '';
    if (!message) {
        result += 'An error happens executing the operation ' + operationType.name;
        if (cause) {
            result += ': ';
            if (cause instanceof OperationExecutionError) {
                result += cause.simpleMessage;
            }
            else if (cause instanceof Error) {
                result += cause.message;
            }
            else {
                result += cause;
            }
        }
    }
    else {
        result = message;
    }
    result += '\nOperation type: ' + operationType.name;
    result += '\nOperation: ' + OperationExecutionError.stringifyOperation(operation, operationType);
    result += '\nContext: ' + OperationExecutionError.stringifyContext(context, operationType);
    if (cause) {
        if (cause instanceof Error) {
            result += '\n\nCaused by: ' + cause.message;
        }
        else {
            result += '\n\nCaused by: ' + cause;
        }
    }
    return result;
}
