"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImplementedOperationByName = exports.getImplementedOperations = exports.hasOperationImplementation = exports.interceptAnyOperation = exports.filterImplementationsByOperationType = exports.combineExecutors = void 0;
const implementation_1 = require("./implementation");
function combineExecutors(...executors) {
    const implementedOperations = {};
    const implementedOperationTypes = {};
    executors.map((executor) => {
        const privateExecutor = implementation_1._asExecutorPrivate(executor);
        for (let operationName in privateExecutor._implementedOperations) {
            implementedOperations[operationName] = privateExecutor._implementedOperations[operationName];
            implementedOperationTypes[operationName] = privateExecutor._implementedOperationTypes[operationName];
        }
    });
    return implementation_1._implementCombinedExecutor(implementedOperations, implementedOperationTypes);
}
exports.combineExecutors = combineExecutors;
function filterImplementationsByOperationType(executor, filter) {
    const implementedOperations = {};
    const implementedOperationTypes = {};
    const executorPrivate = implementation_1._asExecutorPrivate(executor);
    for (let operationName in executorPrivate._implementedOperations) {
        const impl = executorPrivate._implementedOperations[operationName];
        const op = executorPrivate._implementedOperationTypes[operationName];
        if (filter(op)) {
            implementedOperations[operationName] = impl;
            implementedOperationTypes[operationName] = op;
        }
    }
    return implementation_1._implementCombinedExecutor(implementedOperations, implementedOperationTypes);
}
exports.filterImplementationsByOperationType = filterImplementationsByOperationType;
function interceptAnyOperation(executor, interceptor) {
    const implementedOperations = {};
    const implementedOperationTypes = {};
    const executorPrivate = implementation_1._asExecutorPrivate(executor);
    for (let operationName in executorPrivate._implementedOperations) {
        const impl = executorPrivate._implementedOperations[operationName];
        const op = executorPrivate._implementedOperationTypes[operationName];
        implementedOperations[operationName] = implementation_1._implementInterceptor(op, impl, interceptor);
        implementedOperationTypes[operationName] = op;
    }
    return implementation_1._implementCombinedExecutor(implementedOperations, implementedOperationTypes);
}
exports.interceptAnyOperation = interceptAnyOperation;
function hasOperationImplementation(executor, operationType) {
    const executorPrivate = implementation_1._asExecutorPrivate(executor);
    return operationType.name in executorPrivate._implementedOperations;
}
exports.hasOperationImplementation = hasOperationImplementation;
function getImplementedOperations(executor) {
    const executorPrivate = implementation_1._asExecutorPrivate(executor);
    return Object.values(executorPrivate._implementedOperationTypes);
}
exports.getImplementedOperations = getImplementedOperations;
function getImplementedOperationByName(executor, operationName) {
    const executorPrivate = implementation_1._asExecutorPrivate(executor);
    return executorPrivate._implementedOperationTypes[operationName];
}
exports.getImplementedOperationByName = getImplementedOperationByName;
