"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._implementInterceptor = exports._implementCombinedExecutor = exports._implementExecutor = exports._asExecutorPrivate = void 0;
const errors_1 = require("./errors");
function _asExecutorPrivate(executor) {
    return executor;
}
exports._asExecutorPrivate = _asExecutorPrivate;
function _implementExecutor(operationType, executor) {
    function executorImpl(operation, context, type) {
        if (operationType !== type) {
            throw new errors_1.OperationExecutionError(operation, context, type, 'No handler found for the operation: ' + type.name);
        }
        try {
            return executor(operation, context, operationType);
        }
        catch (error) {
            if (error instanceof errors_1.OperationExecutionError) {
                if (error.sameContent(operation, context, operationType)) {
                    throw error;
                }
                throw new errors_1.OperationExecutionError(operation, context, operationType, undefined, error);
            }
            else if (errors_1.PublicError.isPublicError(error)) {
                throw error;
            }
            else {
                throw new errors_1.OperationExecutionError(operation, context, operationType, undefined, error);
            }
        }
    }
    const impl = executorImpl;
    impl._implementedOperations = {
        [operationType.name]: impl
    };
    impl._implementedOperationTypes = {
        [operationType.name]: operationType
    };
    return impl;
}
exports._implementExecutor = _implementExecutor;
function _implementCombinedExecutor(implementedOperations, implementedOperationTypes) {
    function executorImpl(operation, context, operationType) {
        const impl = implementedOperations[operationType.name];
        if (!impl) {
            throw new errors_1.OperationExecutionError(operation, context, operationType, 'No handler found for the operation: ' + operationType.name);
        }
        return impl(operation, context, operationType);
    }
    const impl = executorImpl;
    impl._implementedOperations = implementedOperations;
    impl._implementedOperationTypes = implementedOperationTypes;
    return impl;
}
exports._implementCombinedExecutor = _implementCombinedExecutor;
function _implementInterceptor(operationType, impl, interceptor) {
    function executorImpl(operation, context, opType) {
        try {
            return interceptor(operation, context, opType, impl);
        }
        catch (error) {
            if (error instanceof errors_1.OperationExecutionError) {
                if (error.sameContent(operation, context, opType)) {
                    throw error;
                }
                throw new errors_1.OperationExecutionError(operation, context, opType, undefined, error);
            }
            else if (errors_1.PublicError.isPublicError(error)) {
                throw error;
            }
            else {
                throw new errors_1.OperationExecutionError(operation, context, opType, undefined, error);
            }
        }
    }
    const intercepted = executorImpl;
    intercepted._implementedOperations = {
        [operationType.name]: intercepted
    };
    intercepted._implementedOperationTypes = {
        [operationType.name]: operationType
    };
    return intercepted;
}
exports._implementInterceptor = _implementInterceptor;
